# -*- coding: utf-8 -*-

"""Swahili language module."""

from .base import AbstractNumber


class Number(AbstractNumber):

    ZERO = 'sifuri'
    NEGATIVE = 'hasi'
    ONES = ['', 'moja', 'mbili', 'tatu', 'nne', 'tano', 'sita', 'saba', 'nane', 'tisa']
    TENS = [
        '', 'kumi', 'ishirini', 'thelathini', 'arobaini', 'hamsini', 'sitini',
        'sabini', 'themanini', 'tisini'
    ]
    HUNDREDS = [
        '', 'mia moja', 'mia mbili', 'mia tatu', 'mia nne', 'mia tano',
        'mia sita', 'mia saba', 'mia nane', 'mia tisa'
    ]
    RANKS = [
        '', 'elfu', 'milioni', 'bilioni', 'trilioni', 'kuadrilioni',
        'kuintilioni', 'seksitilioni', 'septilioni', 'oktilioni', 'nonilioni',
        'desilioni', 'anidesilioni', 'dodesilioni', 'tradesilioni',
        'kuatuordesilion', 'kuindesilioni', 'seksidesilioni',
        'septendesilioni', 'oktodesilioni', 'novemdesilioni', 'vijintilioni'
    ]
    CONJUNCTION = 'na'
    POINT = 'nukta'

    def __init__(self, number, **kwargs):
        super(Number, self).__init__(number)

        kwargs = kwargs or {}
        self.use_lakh = kwargs.get('use_lakh', False)

    @classmethod
    def digits_to_words(cls, number):
        """Get words for each digit in a number

        Args:
            number (numeric): A whole number

        Returns:
            str: words for each digit in a number.

        """
        words = []

        digits = str(number)

        for i in digits:
            if int(i) == 0:
                words.append(cls.ZERO)
            else:
                words.append(cls.ONES[int(i)])

        return ' '.join(words)

    @classmethod
    def hundreds_to_words(cls, number):
        """Get a whole number within range less than one thousand in words.

        Args:
            number (numeric): a numeric value to be converted

        Returns
            str: numeric value in words.

        """
        words = []
        number = int(number) % 1000

        if number >= 100:
            hundred = number // 100
            hundredr = number % 100
            if hundredr:
                ten = hundredr // 10
                one = hundredr % 10
                words.append(cls.HUNDREDS[hundred])
                if ten:
                    words.append(cls.CONJUNCTION)
                    words.append(cls.TENS[ten])
                if one:
                    words.append(cls.CONJUNCTION)
                    words.append(cls.ONES[one])
            else:
                words.append(cls.HUNDREDS[hundred])

        elif 100 > number >= 10:
            ten = number // 10
            one = number % 10
            words.append(cls.TENS[ten])

            if one:
                words.append(cls.CONJUNCTION)
                words.append(cls.ONES[one])

        else:
            words.append(cls.ONES[number])

        if not words:
            return ''

        return ' '.join(words)

    @classmethod
    def short_scale_to_words(cls, number, use_lakh=False):
        """Get a value within a short scale range of a number in words.

        Args:
            number (numeric): An integer or a numeric string to be converted.

            use_lakh (bool): Use Lakh (Laki) numbering system.
                Defaults to False.

        Returns:
            str: numeric value within numbers short scale in words.

        """
        number = int(number)
        words = []

        rank = cls.get_short_scale(number)

        hundred = number // 10**(3*rank)
        if rank == 1 and hundred >= 100 and use_lakh:
            laki = hundred // 100
            lakir = hundred % 100
            words += ['laki', cls.ONES[laki]]
            if lakir:
                words += [cls.CONJUNCTION, 'elfu', cls.hundreds_to_words(lakir)]
        else:
            words = [cls.RANKS[rank], cls.hundreds_to_words(hundred)]

        return ' '.join(words)

    @classmethod
    def short_scale_to_words_r(cls, number):
        """Get a value within the short scale range of a number in words
        with words representing scale added at the end.

        Args:
            number (numeric): An integer or a numeric string to be converted.

            use_lakh (bool): Use Lakh (Laki) numbering system.
                             Defaults to False.

        Returns:
            str: numeric value within numbers short scale in words.

        """
        number = int(number)
        words = []
        rank = cls.get_short_scale(number)
        words = [cls.hundreds_to_words(number // 10**(3*rank)), cls.RANKS[rank]]

        return ' '.join(words)

    def to_words(self):
        """Get words representing the instance number.

        Returns:
            str: number in words.

        """
        words = []
        cls = self.__class__
        quotient = abs(int(self.quotient))

        if self.is_negative:
            words.append(cls.NEGATIVE)

        if quotient == 0:
            words.append(cls.ZERO)

        elif quotient < 1000:
            words.append(cls.hundreds_to_words(self.quotient))

        else:
            if quotient % 1000:
                if (quotient % 1000) < 100:
                    conjunction = cls.CONJUNCTION
                else:
                    conjunction = ','
            else:
                conjunction = None

            while quotient >= 1000:
                rank = self.get_short_scale(quotient)
                _next = quotient - (quotient // 10**(3*rank)) * (10**(3*rank))
                if (100000 <= quotient < 1000000) and not self.use_lakh:
                    words.append(cls.short_scale_to_words_r(quotient))
                else:
                    words.append(cls.short_scale_to_words(quotient, self.use_lakh))

                if _next >= 1000:
                    words.append(',')  # TODO: should allow either ',' or 'na'

                quotient = _next
            else:
                if conjunction:
                    words += [conjunction, cls.hundreds_to_words(quotient)]

        if self.is_decimal:
            words += [cls.POINT, cls.digits_to_words(self.fraction)]

        return ' '.join(words).replace(' , ', ', ')



# -*- coding: utf-8 -*-

"""tarakimu console script."""

import click
from .numtowords import numtowords


@click.group()
def cli():
    pass


cli.add_command(numtowords)


if __name__ == "__main__":
    cli()



import pytest


@pytest.fixture(scope="module")
def numwords():
    return {
        'sw': {
            '88': 'themanini na nane',
            '-88': 'hasi themanini na nane',
            '1001.1': 'elfu moja na moja nukta moja',
            100: 'mia moja',
            4000000000: 'bilioni nne',
            1111111111111111111111111111111111: (
                'desilioni moja, nonilioni mia moja na kumi na moja, '
                'oktilioni mia moja na kumi na moja, septilioni mia moja na '
                'kumi na moja, seksitilioni mia moja na kumi na moja, '
                'kuintilioni mia moja na kumi na moja, kuadrilioni mia moja '
                'na kumi na moja, trilioni mia moja na kumi na moja, bilioni '
                'mia moja na kumi na moja, milioni mia moja na kumi na moja, '
                'mia moja na kumi na moja elfu, mia moja na kumi na moja'),
            '1111111111111111111111111111111111.11': (
                'desilioni moja, nonilioni mia moja na kumi na moja, '
                'oktilioni mia moja na kumi na moja, septilioni mia moja na '
                'kumi na moja, seksitilioni mia moja na kumi na moja, '
                'kuintilioni mia moja na kumi na moja, kuadrilioni mia moja '
                'na kumi na moja, trilioni mia moja na kumi na moja, bilioni '
                'mia moja na kumi na moja, milioni mia moja na kumi na moja, '
                'mia moja na kumi na moja elfu, mia moja na kumi na moja '
                'nukta moja moja')
        },
        'en': {
            '88': 'eighty eight',
            '-88': 'negative eighty eight',
            '1001.1': 'one thousand and one point one',
            100: 'one hundred',
            2012: 'two thousand and twelve',
            112: 'one hundred and twelve',
            100103: 'one hundred thousand, one hundred and three',
            3000000000: 'three billion',
            11111111111111111111: (
                'eleven quintillion, one hundred and eleven quadrillion, one '
                'hundred and eleven trillion, one hundred and eleven billion, '
                'one hundred and eleven million, one hundred and eleven '
                'thousand, one hundred and eleven'),
            '11111111111111111111.11': (
                'eleven quintillion, one hundred and eleven quadrillion, one '
                'hundred and eleven trillion, one hundred and eleven billion, '
                'one hundred and eleven million, one hundred and eleven '
                'thousand, one hundred and eleven point one one')
        }
    }


@pytest.fixture(scope="module")
def invalid_nums():
    return ['88a', '108,000']



# -*- coding: utf-8 -*-

import importlib


def get_lang(lang):
    """Get module for a give language.

    Args:
        lang (str): code representing the language.

        CUrrently the supported laguages are 'sw' and 'en' for Swahili and English.
    """
    return importlib.import_module('tarakimu.lang.{}'.format(lang))


def num_to_words(number, lang='sw', **kwargs):
    """Get number in words.

    Args:
        number (int): a numerical value to be converted.

        \*\*kwargs: a optional keyword arguments
            use_lakh (bool): use lakh numbering system in Swahili.

    Returns:
        str: words reprenting a number.
    """
    return get_lang(lang).Number(number, **kwargs).to_words()



# -*- coding: utf-8 -*-

"""Console script for tarakimu."""

import click
from ..utils import num_to_words


@click.command(name='numtowords')
@click.argument('number', type=str)
@click.option('--lang', '-l',
              type=click.Choice(['en', 'sw']), default='sw',
              help='language code')
@click.option('--use_lakh', '-L',
              is_flag=True,
              help='use lakh numbering format (Swahili only).')
def numtowords(number, lang='sw', use_lakh=False):
    """Convert number to words."""
    click.echo(num_to_words(number, lang=lang, use_lakh=use_lakh))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `tarakimu` CLI."""

from click.testing import CliRunner
from tarakimu import cli


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()

    result = runner.invoke(cli.cli)
    assert result.exit_code == 0
    assert 'cli' in result.output
    assert 'numtowords' in result.output

    help_result = runner.invoke(cli.cli, ['--help'])
    assert help_result.exit_code == 0
    assert '--help' in help_result.output


def test_numtowords(numwords):
    """Test the CLI."""

    runner = CliRunner()

    for lang, samples in numwords.items():
        for n, w in samples.items():
            result = runner.invoke(cli.cli, ['numtowords', '-l', lang, '--', n])
            assert result.exit_code == 0
            assert w == result.output.strip()


def test_invalid_numtowords(invalid_nums):
    """Test the CLI."""

    runner = CliRunner()

    for n in invalid_nums:
        result = runner.invoke(cli.cli, ['numtowords', n])
        assert result.exit_code != 0
        assert result.output == ''



# -*- coding: utf-8 -*-

"""English language module."""

from .base import AbstractNumber


class Number(AbstractNumber):

    ZERO = 'zero'
    NEGATIVE = 'negative'
    ONES = ['', 'one', 'two', 'three', 'four', 'five', 'six', 'seven', 'eight', 'nine']
    TEENS = [
        '', 'eleven', 'twelve', 'thirteen', 'fourteen', 'fifteen', 'sixteen',
        'seventeen', 'eighteen', 'nineteen'
    ]
    TENS = [
        '', 'ten', 'twenty', 'thirty', 'fourty', 'fifty', 'sixty', 'seventy',
        'eighty', 'ninety'
    ]
    HUNDREDS = [
        '', 'one hundred', 'two hundred', 'three hundred', 'four hundred',
        'five hundred', 'six hundred', 'seven hundred', 'eight hundred',
        'nine hundred'
    ]
    RANKS = [
        '', 'thousand', 'million', 'billion', 'trillion', 'quadrillion',
        'quintillion', 'sextillion', 'septillion', 'octillion', 'nonillion',
        'decillion', 'undecillion', 'duodecillion', 'tredecillion',
        'quattuordecillion', 'quindecillion', 'sexdecillion',
        'septendecillion', 'octodecillion', 'novemdecillion', 'vigintillion'
    ]
    CONJUNCTION = 'and'
    POINT = 'point'

    @classmethod
    def digits_to_words(cls, number, **kwargs):
        """Get words for each digit in a number

        Args:
            number (numeric): A whole number

        Returns:
            str: words for each digit in a number.

        """
        words = []

        digits = str(number)

        for i in digits:
            if int(i) == 0:
                words.append(cls.ZERO)
            else:
                words.append(cls.ONES[int(i)])

        return ' '.join(words)

    @classmethod
    def hundreds_to_words(cls, number):
        """Get a whole number within range less than one thousand in words.

        Args:
            number (numeric): a numeric value to be converted

        Returns
            str: numeric value in words.

        """
        words = []
        number = int(number) % 1000

        if number >= 100:
            hundred = number // 100
            hundredr = number % 100
            if hundredr:
                ten = hundredr // 10
                one = hundredr % 10
                words.append(cls.HUNDREDS[hundred])

                ten_ = (number % 100)
                if 20 > ten_ > 10:
                    words.append(cls.CONJUNCTION)
                    words.append(cls.TEENS[ten_ - 10])
                elif ten:
                    words.append(cls.CONJUNCTION)
                    words.append(cls.TENS[ten])
                    if one:
                        words.append(cls.ONES[one])
                else:
                    words += [cls.CONJUNCTION, cls.ONES[one]]
            else:
                words.append(cls.HUNDREDS[hundred])

        elif 100 > number >= 10:
            if 20 > number > 10:
                teen = number - 10
                words.append(cls.TEENS[teen])
            else:
                ten = number // 10
                one = number % 10
                words.append(cls.TENS[ten])

                if one and not ten:
                    words.append(cls.CONJUNCTION)
                elif one:
                    words.append(cls.ONES[one])

        else:
            words.append(cls.ONES[number])

        if not words:
            return ''

        return ' '.join(words)

    @classmethod
    def short_scale_to_words(cls, number, use_lakh=False):
        """Get a value within a short scale range of a number in words.

        Args:
            number (numeric): An integer or a numeric string to be converted.

        Returns:
            str: numeric value within numbers short scale in words
        """
        words = []
        number = int(number)
        rank = cls.get_short_scale(number)
        hundred = number // 10**(3*rank)
        words = [cls.hundreds_to_words(hundred), cls.RANKS[rank]]

        return ' '.join(words)

    def to_words(self):
        """Get words representing the instance number.

        Returns:
            str: number in words.

        """
        words = []
        cls = self.__class__
        quotient = abs(int(self.quotient))

        if self.is_negative:
            words.append(cls.NEGATIVE)

        if quotient == 0:
            words.append(cls.ZERO)

        elif quotient < 1000:
            words.append(cls.hundreds_to_words(self.quotient))

        else:
            if quotient % 1000:
                if (quotient % 1000) < 100:
                    conjunction = cls.CONJUNCTION
                else:
                    conjunction = ','
            else:
                conjunction = None

            while quotient >= 1000:
                rank = self.get_short_scale(quotient)
                _next = quotient - (quotient // 10**(3*rank)) * (10**(3*rank))

                words.append(cls.short_scale_to_words(quotient))

                if _next >= 1000:
                    words.append(',')

                quotient = _next
            else:
                if conjunction:
                    words += [conjunction, cls.hundreds_to_words(quotient)]

        if self.is_decimal:
            words += [cls.POINT, cls.digits_to_words(self.fraction)]

        return ' '.join(words).replace(' , ', ', ')



# -*- coding: utf-8 -*-

"""Absarct language module."""


class AbstractNumber(object):

    def __init__(self, number, **kwargs):
        self.number = str(number)

    @property
    def quotient(self):
        """str: number quotient."""
        return self.number.split('.')[0].replace('-', '')

    @property
    def fraction(self):
        """str: fraction part of instance number."""
        f = self.number.split('.')

        if len(f) < 2:
            return ''

        return f[1]

    @property
    def is_decimal(self):
        """bool: instance number is decimal."""
        return len(self.number.split('.')) > 1

    @property
    def is_negative(self):
        """bool: instance number is negative."""
        return self.number.startswith('-')

    @staticmethod
    def get_short_scale(number):
        """Get short scale index of a number.

        Short scale system groups numbers based on powers of one thousand.

        Args:
            number (numeric): a numeric value to be ranked.

        Returns
            int: short scale index of a number, starting from 0.

        """
        rank = 0
        number = int(number)
        while number >= 1000:
            rank += 1
            number = number // 1000
        return rank



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `tarakimu.num_to_words`."""

import pytest
from tarakimu import num_to_words


def test_num_to_words(numwords):
    """test num_to_words returns to right words for valid numbers"""
    for lang, samples in numwords.items():
        for n, w in samples.items():
            result = num_to_words(n, lang)
            assert result == w


def test_invalid_num_to_words(invalid_nums):
    """test num_to_words returns to right words for invalid numbers"""
    for n in invalid_nums:
        with pytest.raises(ValueError):
            num_to_words(n)
